@@ -74,7 +74,7 @@ typedef struct {
 	GtkTooltips *tooltips;
 	pilot_properties properties; /* = { NULL }; */
 	state curstate;
-	GtkWidget *pixmap; 
+	GtkWidget *image; 
 	GtkWidget *dialogWindow; 
 	GtkWidget *operationDialogWindow;
 	GtkWidget *pb;
@@ -113,6 +113,7 @@ static void pilot_draw (PilotApplet*);
 
 static void pilot_execute_program (PilotApplet*, const gchar *);
 static gchar *pick_pilot (PilotApplet *self);
+static void response_cb (GtkDialog *dialog, gint id, gpointer data);
 
 static gboolean timeout (PilotApplet *self);
 
@@ -132,9 +133,10 @@ gpilotd_connect_cb (GnomePilotClient *client,
 	gchar *buf;
 	PilotApplet *applet = PILOT_APPLET (user_data);
 
-	gtk_tooltips_set_tip (applet->tooltips, GTK_WIDGET(applet) ,_("Synchronizing..."), NULL);
+	gtk_tooltips_set_tip (applet->tooltips, GTK_WIDGET(applet->applet),
+			("Synchronizing..."), NULL);
   
-	if (!GTK_WIDGET_REALIZED (applet->pixmap)) {
+	if (!GTK_WIDGET_REALIZED (applet->image)) {
 		g_warning ("! realized");
 		return;
 	}
@@ -144,6 +146,8 @@ gpilotd_connect_cb (GnomePilotClient *client,
 	if (applet->properties.popups == FALSE) return;
 
 	if (applet->progressDialog == NULL) {
+		gnome_window_icon_set_default_from_file (
+				GNOME_ICONDIR "/sync_icon.png");
 		GladeXML *xml              = glade_xml_new (applet->glade_file,"ProgressDialog",NULL);
 		applet->progressDialog       = glade_xml_get_widget (xml,"ProgressDialog");
 		applet->sync_label           = glade_xml_get_widget (xml,"sync_label");
@@ -151,12 +155,13 @@ gpilotd_connect_cb (GnomePilotClient *client,
 		applet->overall_progress_bar = glade_xml_get_widget (xml,"overall_progress_bar");
 		applet->conduit_progress_bar = glade_xml_get_widget (xml,"conduit_progress_bar");
 		applet->cancel_button        = glade_xml_get_widget (xml,"cancel_button");
+		applet->message_buffer       = gtk_text_view_get_buffer(
+				GTK_TEXT_VIEW(applet->message_area));
 
 		gtk_signal_connect (GTK_OBJECT (applet->cancel_button),"clicked",
 				   GTK_SIGNAL_FUNC (cancel_cb),applet);
 	} else {
-		gtk_text_buffer_place_cursor (applet->message_buffer,0);
-		gtk_text_buffer_set_text (applet->message_buffer, "", 0);
+		gtk_text_buffer_set_text (applet->message_buffer, "", -1);
 	}
 
 	gtk_widget_set_sensitive (applet->cancel_button, FALSE);
@@ -184,7 +189,8 @@ gpilotd_disconnect_cb (GnomePilotClient *client,
 		       gpointer user_data)
 {
 	PilotApplet *applet = PILOT_APPLET (user_data);
-	gtk_tooltips_set_tip (applet->tooltips, GTK_WIDGET(applet) ,_("Ready to synchronize"), NULL);
+	gtk_tooltips_set_tip (applet->tooltips, GTK_WIDGET(applet->applet),
+			("Ready to synchronize"), NULL);
 
 	applet->curstate = WAITING;
 	pilot_draw (applet);
@@ -194,6 +200,60 @@ gpilotd_disconnect_cb (GnomePilotClient *client,
 	}
 }
 
+/* taken from http://live.gnome.org/GnomeLove/PanelAppletTutorial */
+static void
+applet_back_change (PanelApplet			*a,
+		    PanelAppletBackgroundType	type,
+		    GdkColor			*color,
+		    GdkPixmap			*pixmap,
+		    PilotApplet			*applet) 
+{
+        GtkRcStyle *rc_style;
+        GtkStyle *style;
+
+        /* reset style */
+        gtk_widget_set_style (GTK_WIDGET (applet->applet), NULL);
+        rc_style = gtk_rc_style_new ();
+        gtk_widget_modify_style (GTK_WIDGET (applet->applet), rc_style);
+        gtk_rc_style_unref (rc_style);
+
+        switch (type) {
+                case PANEL_COLOR_BACKGROUND:
+                        gtk_widget_modify_bg (GTK_WIDGET (applet->applet),
+                                        GTK_STATE_NORMAL, color);
+                        break;
+
+                case PANEL_PIXMAP_BACKGROUND:
+                        style = gtk_style_copy (GTK_WIDGET (
+						applet->applet)->style);
+                        if (style->bg_pixmap[GTK_STATE_NORMAL])
+                                g_object_unref
+                                        (style->bg_pixmap[GTK_STATE_NORMAL]);
+                        style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref
+                                (pixmap);
+                        gtk_widget_set_style (GTK_WIDGET (applet->applet),
+                                        style);
+                        g_object_unref (style);
+                        break;
+
+                case PANEL_NO_BACKGROUND:
+                default:
+                        break;
+        }
+
+}
+
+static void gpilotd_scroll_to_insert_mark(PilotApplet *applet)
+{
+	gtk_text_view_scroll_to_mark(
+			GTK_TEXT_VIEW(applet->message_area),
+       			gtk_text_buffer_get_insert(applet->message_buffer),
+			0.2,
+			FALSE,
+			0.0,
+			0.0);
+}
+
 static void 
 gpilotd_request_completed (GnomePilotClient *client,
 			   const gchar *id,
@@ -208,6 +268,7 @@ gpilotd_request_completed (GnomePilotClient *client,
 		gtk_text_buffer_insert_at_cursor (applet->message_buffer,txt,-1);
 		g_free (txt);
 
+		gpilotd_scroll_to_insert_mark(applet);
 	}
 
 /*	show_message_dialog ("Pilot %s has completed request %ld",
@@ -233,6 +294,7 @@ gpilotd_conduit_start (GnomePilotClient* client,
 		txt=g_strdup_printf (_("%s: Started\n"),conduit);
 		gtk_text_buffer_insert_at_cursor (applet->message_buffer,txt,-1);
 		g_free (txt);
+		gpilotd_scroll_to_insert_mark(applet);
 	}
 }
 
@@ -253,6 +315,7 @@ gpilotd_conduit_end (GnomePilotClient* client,
 		gtk_text_buffer_insert_at_cursor (applet->message_buffer,txt,-1);
 		g_free (txt);
 		gtk_progress_set_format_string (GTK_PROGRESS (applet->conduit_progress_bar), "");
+		gpilotd_scroll_to_insert_mark(applet);
 	}
 }
 
@@ -312,6 +375,7 @@ gpilotd_conduit_message (GnomePilotClient* client,
 		gchar *txt=g_strdup_printf ("%s: %s\n",conduit,message);
 		gtk_text_buffer_insert_at_cursor (applet->message_buffer,txt,-1);
 		g_free (txt);
+		gpilotd_scroll_to_insert_mark(applet);
 	}
 	g_main_iteration (FALSE);
 }
@@ -328,6 +392,7 @@ gpilotd_daemon_message (GnomePilotClient* client,
 		gchar *txt=g_strdup_printf ("%s\n", message);
 		gtk_text_buffer_insert_at_cursor (applet->message_buffer,txt,-1);
 		g_free (txt);
+		gpilotd_scroll_to_insert_mark(applet);
 	}
 	g_main_iteration (FALSE);
 }
@@ -343,6 +408,7 @@ gpilotd_daemon_error (GnomePilotClient* client,
 		gchar *txt=g_strdup_printf ("Error: %s\n", message);
 		gtk_text_buffer_insert_at_cursor (applet->message_buffer,txt,-1);
 		g_free (txt);
+		gpilotd_scroll_to_insert_mark(applet);
 	}
 }
 
@@ -350,7 +416,7 @@ static void
 handle_client_error (PilotApplet *self)
 {
 	if (self->curstate == SYNCING) {
-		show_warning_dialog (self, _("Pilot is currently synchronizing.\nPlease wait for it to finish."));
+		show_warning_dialog (self, _("PDA is currently synchronizing.\nPlease wait for it to finish."));
 	} else {
 		self->curstate=INITIALISING;
 		gtk_tooltips_set_tip (self->tooltips, GTK_WIDGET (self->applet),_("Not connected. Please restart daemon."), NULL);	
@@ -368,12 +434,15 @@ about_cb(BonoboUIComponent *uic, PilotApplet *pilot, const gchar *verbname)
 				  "JP Rosevear <jpr@ximian.com>", 
 				  "Chris Toshok <toshok@ximian.com>",
 				  "Frank Belew <frb@ximian.com>",
+				  "Matt Davey <mcdavey@mrao.cam.ac.uk>",
 				  NULL};
 
+	gnome_window_icon_set_default_from_file (
+				GNOME_ICONDIR "/sync_icon.png");
 	about = gnome_about_new (_("gnome-pilot applet"), 
 				 VERSION,
-				 _("Copyright 2000-2003 Free Software Foundation, Inc."),
-				 _("A dnd and gui monitor for gpilotd.\n"),
+				 _("Copyright 2000-2006 Free Software Foundation, Inc."),
+				 _("A PalmOS PDA monitor.\n"),
 				 (const gchar**)authors,
 				 NULL,
 				 "Translations by the GNOME Translation Project",
@@ -381,8 +450,10 @@ about_cb(BonoboUIComponent *uic, PilotApplet *pilot, const gchar *verbname)
 
 	gtk_window_set_wmclass (
 		GTK_WINDOW (about), "pilot", "Pilot");
-	gnome_window_icon_set_from_file (GTK_WINDOW (about),
-					 GNOME_ICONDIR "/sync_icon.png");
+
+	/* multi-headed displays... */
+	gtk_window_set_screen (GTK_WINDOW (about),
+		gtk_widget_get_screen (GTK_WIDGET (pilot->applet)));
 
 	gtk_widget_show (about);
 
@@ -423,6 +494,7 @@ properties_cb (BonoboUIComponent *uic, gpointer user_data, const gchar *verbname
 	GtkWidget *button, *entry, *dialog;
 	GladeXML *xml;
 
+	gnome_window_icon_set_default_from_file (GNOME_ICONDIR "/sync_icon.png");
 	xml =glade_xml_new (self->glade_file,"PropertiesDialog", NULL);
 	dialog=glade_xml_get_widget (xml,"PropertiesDialog");
 	
@@ -440,11 +512,12 @@ properties_cb (BonoboUIComponent *uic, gpointer user_data, const gchar *verbname
 			    GTK_SIGNAL_FUNC (toggle_notices_cb),
 			    self);
 
-	gtk_signal_connect (GTK_OBJECT (dialog), "response",
-			    GTK_SIGNAL_FUNC (gtk_widget_hide),
-			    NULL);
+	g_signal_connect (GTK_OBJECT (dialog), "response",
+			  G_CALLBACK (response_cb),
+			  NULL);
 
-	gnome_window_icon_set_from_file (GTK_WINDOW (dialog), GNOME_ICONDIR "/sync_icon.png");
+	gtk_window_set_screen (GTK_WINDOW (dialog),
+		gtk_widget_get_screen (GTK_WIDGET (self->applet)));
 
 	gtk_widget_show_all (dialog);
 }
@@ -452,10 +525,70 @@ properties_cb (BonoboUIComponent *uic, gpointer user_data, const gchar *verbname
 static void
 help_cb(BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
 {
-	/* FIXME */
+	PilotApplet *applet = user_data;
+	GError *error = NULL;
+
+	gnome_help_display_on_screen  ("gnome-pilot", "pilot-applet",
+				       (gtk_widget_get_screen(GTK_WIDGET (applet->applet))),
+				       &error);
+
+	if (error)
+	{
+		GtkWidget *message_dialog;
+		message_dialog = gtk_message_dialog_new (NULL, 0,
+							 GTK_MESSAGE_ERROR,
+							 GTK_BUTTONS_OK,
+							 _("There was an error displaying help: %s"),
+							 error->message);
+		g_error_free (error);
+		gtk_window_set_screen (GTK_WINDOW (message_dialog),
+				       gtk_widget_get_screen (GTK_WIDGET (applet->applet)));
+		gtk_dialog_run (GTK_DIALOG (message_dialog));
+		gtk_widget_destroy (message_dialog);
+	}
+
 	return;
 }
 
+static void
+pref_help_cb(GtkDialog *dialog)
+{
+	GError *error = NULL;
+	gnome_help_display_on_screen  ("gnome-pilot", NULL /* FIXME Add section id later */,
+				       gtk_widget_get_screen (GTK_WIDGET (dialog)),
+				       &error);
+
+	if (error)
+	{
+		GtkWidget *message_dialog;
+		message_dialog = gtk_message_dialog_new (NULL, 0,
+							 GTK_MESSAGE_ERROR,
+							 GTK_BUTTONS_OK,
+							 _("There was an error displaying help: %s"),
+							 error->message);
+
+		g_error_free (error);
+		gtk_window_set_screen (GTK_WINDOW (message_dialog),
+				       gtk_widget_get_screen (GTK_WIDGET (dialog)));
+				       gtk_dialog_run (GTK_DIALOG (message_dialog));
+		gtk_widget_destroy (message_dialog);
+	}
+	return;
+}
+
+static void
+response_cb (GtkDialog *dialog, gint id, gpointer data)
+{
+
+	if (id == GTK_RESPONSE_HELP)
+	{
+		pref_help_cb (dialog);
+		return;
+	}
+
+	gtk_widget_hide (GTK_WIDGET (dialog));
+}
+
 
 static void
 complete_restore (GnomePilotClient* client, const gchar *id, 
@@ -618,7 +751,7 @@ restart_cb (BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
 	PilotApplet *applet = user_data;
 
 	if (applet->curstate == SYNCING) {
-		if ( gnome_dialog_run_and_close (GNOME_DIALOG (gnome_question_dialog (_("Pilot sync is currently in progress.\nAre you sure you want to restart daemon?"),NULL,NULL))) !=0 ) {
+		if ( gnome_dialog_run_and_close (GNOME_DIALOG (gnome_question_dialog (_("PDA sync is currently in progress.\nAre you sure you want to restart daemon?"),NULL,NULL))) !=0 ) {
 			return;
 		}
 		if (applet->progressDialog!=NULL) {
@@ -664,6 +797,12 @@ static void
 install_popup_menu (PilotApplet *self, gboolean on_off)
 {
 	char *menu_xml;
+	/* ensure the label strings are extracted by intltool.  This list of
+	 * strings should match the list in the menu_template below
+	 */
+#define FOO_STRING _("Restore...") _("Restart") _("Last log...") \
+           _("Preferences...") _("Help") _("About")
+#undef FOO_STRING
 	const char *menu_template = 
 		"<popup name=\"button3\">"
 		"<menuitem name=\"Restore\" verb=\"Restore\" _label=\"Restore...\""
@@ -679,7 +818,7 @@ install_popup_menu (PilotApplet *self, gboolean on_off)
 		"          pixtype=\"stock\" pixname=\"gtk-properties\"/>"
 		"<menuitem name=\"Item 2\" verb=\"Help\" _label=\"Help\""
 		"          pixtype=\"stock\" pixname=\"gtk-help\"/>"
-		"<menuitem name=\"Item 3\" verb=\"About\" _label=\"About...\""
+		"<menuitem name=\"Item 3\" verb=\"About\" _label=\"About\""
 		"          pixtype=\"stock\" pixname=\"gnome-stock-about\"/>"
 		"</popup>";
 
@@ -740,9 +879,11 @@ gpilotd_conduit_error (GnomePilotClient* client,
 							  iter,
 							  txt,-1,
 							  "foreground-gdk", 
-							  applet->errorColor);
+							  applet->errorColor,
+							  NULL);
 		g_free (txt);
 		gtk_text_iter_free (iter);
+		gpilotd_scroll_to_insert_mark(applet);
 	}
 }
 
@@ -805,12 +946,12 @@ file_size (char *name)
 static void 
 pilot_draw (PilotApplet *self)
 {
-	if (!GTK_WIDGET_REALIZED (self->pixmap)) {
+	if (!GTK_WIDGET_REALIZED (self->image)) {
 		g_warning ("pilot_draw ! realized");
 		return;
 	}
 
-	gnome_pixmap_load_file (GNOME_PIXMAP (self->pixmap), pixmaps[self->curstate]);
+	gtk_image_set_from_file(GTK_IMAGE(self->image), pixmaps[self->curstate]);
   
 }
 
@@ -1027,23 +1168,10 @@ pick_pilot (PilotApplet *self)
 static void
 pilot_execute_program (PilotApplet *self, const gchar *str)
 {
-	int err = -1;
-	int argc=0;
-	gchar **targv;
-	
 	g_return_if_fail (str != NULL);
 
-	targv=g_strsplit (str," ",0);
-	while (targv[argc]) argc++;
-	g_message ("pilot_execute_program (%s)",str);
-	err = gnome_execute_async (NULL,argc,targv);
-	if (err==-1) {
+	if (!g_spawn_command_line_async (str, NULL))
 		show_warning_dialog (self, _("Execution of %s failed"),str);
-	} else {
-		sleep (1);
-	}
-
-	g_strfreev (targv);
 }
 
 static gint 
@@ -1078,6 +1206,66 @@ pilot_timeout (gpointer data)
 }
 */
 
+/* Copied from glib 2.6 so we can work with older GNOME's */
+static gchar **
+extract_uris (const gchar *uri_list)
+{
+	GSList *uris, *u;
+	const gchar *p, *q;
+	gchar **result;
+	gint n_uris = 0;
+
+	uris = NULL;
+
+	p = uri_list;
+
+	/* We don't actually try to validate the URI according to RFC
+	 * 2396, or even check for allowed characters - we just ignore
+	 * comments and trim whitespace off the ends.  We also
+	 * allow LF delimination as well as the specified CRLF.
+	 *
+	 * We do allow comments like specified in RFC 2483.
+	 */
+	while (p)
+	{
+		if (*p != '#')
+		{
+			while (g_ascii_isspace (*p))
+				p++;
+
+			q = p;
+			while (*q && (*q != '\n') && (*q != '\r'))
+				q++;
+
+			if (q > p)
+			{
+				q--;
+				while (q > p && g_ascii_isspace (*q))
+					q--;
+
+				if (q > p)
+				{
+					uris = g_slist_prepend (uris, g_strndup (p, q - p + 1));
+					n_uris++;
+				}
+			}
+		}
+		p = strchr (p, '\n');
+		if (p)
+			p++;
+	}
+
+	result = g_new (gchar *, n_uris + 1);
+
+	result[n_uris--] = NULL;
+	for (u = uris; u; u = u->next)
+		result[n_uris--] = u->data;
+
+	g_slist_free (uris);
+
+	return result;
+}
+
 static void
 dnd_drop_internal (GtkWidget        *widget,
 		   GdkDragContext   *context,
@@ -1091,7 +1279,6 @@ dnd_drop_internal (GtkWidget        *widget,
 	gchar *pilot_id;
 	gint  file_handle;
 	PilotApplet *self = PILOT_APPLET(data);
-
 	gchar **names;
 	int idx;
 
@@ -1100,28 +1287,22 @@ dnd_drop_internal (GtkWidget        *widget,
 	switch (info)
 	{
 	case TARGET_URI_LIST:
-		names = g_strsplit ((char *)selection_data->data, "\n", -1);
+		names = extract_uris (selection_data->data);
 		for (idx=0; names[idx]; idx++) {
-			char *ptr=names[idx];
-			while (iscntrl (ptr[strlen(ptr)-1])) {
-				ptr[strlen(ptr)-1] = 0;
-			}
-			if (ptr) {
-				/*
-				  if (strstr (file_type (ptr),"text")) {
-				  if (file_size (ptr)>properties.text_limit)
-				  g_message ("installing textfile as Doc (not implemented)");
+			/*
+			  if (strstr (file_type (ptr),"text")) {
+			  if (file_size (ptr)>properties.text_limit)
+			  g_message ("installing textfile as Doc (not implemented)");
 				  else
 				  g_message ("installing textfile as Memo (not implemented)");
 				  }
-				*/
-				g_message (_("installing \"%s\" to \"%s\""), ptr, pilot_id);
-				if (pilot_id) {
-					gnome_pilot_client_install_file (self->gpc,pilot_id,
-									ptr,
-									GNOME_Pilot_PERSISTENT,0,&file_handle);
-				} 
-			}
+			*/
+			g_message (_("installing \"%s\" to \"%s\""), names[idx], pilot_id);
+			if (pilot_id) {
+				gnome_pilot_client_install_file (self->gpc,pilot_id,
+								 names[idx],
+								 GNOME_Pilot_PERSISTENT,0,&file_handle);
+			} 
 		}
 		g_strfreev (names);
 		break;
@@ -1143,7 +1324,6 @@ applet_destroy (GtkWidget *applet, PilotApplet *self)
 static void
 create_pilot_widgets (GtkWidget *widget, PilotApplet *self) 
 { 
-	GtkWidget *event_box; 
 	GtkStyle *style; 
 	int i; 
 
@@ -1166,40 +1346,39 @@ create_pilot_widgets (GtkWidget *widget, PilotApplet *self)
 	self->curstate = INITIALISING; 
 
 	for (i = 0; i < sizeof (pixmaps)/sizeof (pixmaps[0]); i++) 
-		pixmaps[i] = gnome_unconditional_pixmap_file (pixmaps[i]); 
+		pixmaps[i] = gnome_program_locate_file(
+		    NULL, GNOME_FILE_DOMAIN_PIXMAP, pixmaps[i], TRUE, NULL);
 
-	event_box = gtk_event_box_new (); 
+	self->image = gtk_image_new_from_file (pixmaps[self->curstate]); 
 
-	self->pixmap = gnome_pixmap_new_from_file (pixmaps[self->curstate]); 
-
-	gtk_widget_set_events (event_box, gtk_widget_get_events (widget) | 
-			       GDK_BUTTON_PRESS_MASK); 
-	gtk_signal_connect (GTK_OBJECT (event_box), "button-press-event", 
+	gtk_signal_connect (GTK_OBJECT (widget), "button-press-event", 
 			    GTK_SIGNAL_FUNC (pilot_clicked_cb), self); 
 
 
-	gtk_widget_show (self->pixmap);   
-	gtk_container_add (GTK_CONTAINER (event_box), self->pixmap); 
+	gtk_widget_show (self->image);   
 
-	gtk_container_add (GTK_CONTAINER (widget), event_box);
+	gtk_container_add (GTK_CONTAINER (widget), self->image);
 
 	self->timeout_handler_id = gtk_timeout_add (1000,(GtkFunction)timeout,self); 
 
 	gtk_signal_connect (GTK_OBJECT (widget),"destroy", 
 			    GTK_SIGNAL_FUNC (applet_destroy),self); 
 
+	gtk_signal_connect (GTK_OBJECT (widget),"change_background", 
+			    GTK_SIGNAL_FUNC (applet_back_change),self); 
+
 	/* FIXME */
 	/* XXX change_orient */
 	/* XXX change_size */
 
 	/* Now set the dnd features */
-	gtk_drag_dest_set (GTK_WIDGET (self->pixmap), 
+	gtk_drag_dest_set (GTK_WIDGET (self->image), 
 			   GTK_DEST_DEFAULT_ALL, 
 			   drop_types, n_drop_types, 
 			   GDK_ACTION_COPY); 
 
 
-	gtk_signal_connect (GTK_OBJECT (self->pixmap), 
+	gtk_signal_connect (GTK_OBJECT (self->image), 
 			    "drag_data_received", 
 			    GTK_SIGNAL_FUNC (dnd_drop_internal), 
 			    self); 
@@ -1301,7 +1480,7 @@ pilot_applet_factory ( PanelApplet *applet,
 }
 PANEL_APPLET_BONOBO_FACTORY( "OAFIID:GNOME_PilotApplet_Factory",
 			     PANEL_TYPE_APPLET,
-			     "Pilot-Applet",
+			     "gnome-pilot",
 			     "0",
 			     (PanelAppletFactoryCallback) pilot_applet_factory,
 			     NULL)
