@@ -43,7 +43,7 @@ char *pilot_arg=NULL;
 static const struct poptOption options[] = {
 	{"now", 'n', POPT_ARG_NONE, &now, 0, N_("Install immediately"), NULL},
 	{"later", 'l', POPT_ARG_NONE, &later, 0, N_("Install delayed"), NULL},
-	{"pilot", 'p', POPT_ARG_STRING, &pilot_arg, 0, N_("Pilot to install to"), N_("PILOT")},
+	{"pilot", 'p', POPT_ARG_STRING, &pilot_arg, 0, N_("PDA to install to"), N_("PDA")},
 	{NULL, '\0', 0, NULL, 0} /* end the list */
 };
 
@@ -51,7 +51,7 @@ static void
 gpilotd_request_completed (GnomePilotClient *gpc, gchar *pilot_id, gint handle, gpointer data)
 {
 	g_message ("%s completed %d", pilot_id, handle);
-	handles = g_slist_remove (handles,(gpointer)handle);
+	handles = g_slist_remove (handles,GINT_TO_POINTER(handle));
 	if (handles == NULL)
 		gnome_dialog_close (GNOME_DIALOG (dialog));
 }
@@ -62,7 +62,7 @@ cancel_install (GnomeDialog *w, GSList *list)
 	GSList *e;
 
 	for (e=list;e;e = g_slist_next (e)) {
-		gnome_pilot_client_remove_request (gpc,(gint)e->data);  
+		gnome_pilot_client_remove_request (gpc,GPOINTER_TO_INT(e->data));  
 	}
 	g_slist_free (list);
 
@@ -115,8 +115,8 @@ main (int argc, char *argv[])
 	} else {
 		err = gnome_pilot_client_get_pilots (gpc, &pilots);
 		if (err !=GPILOTD_OK || pilots == NULL) {
-			g_warning (_("Unable to get pilot names"));
-			show_warning_dialog (_("Unable to get pilot names"));
+			g_warning (_("Unable to get PDA names"));
+			show_warning_dialog (_("Unable to get PDA names"));
 			exit (1);
 		}
 	}
@@ -139,10 +139,10 @@ main (int argc, char *argv[])
 						       0,
 						       &handle);
 		if (err == GPILOTD_OK) {
-			handles = g_slist_prepend (handles,(gpointer)handle);
-			notfailed = g_slist_prepend (notfailed, args[i]);
+			handles = g_slist_prepend (handles,GINT_TO_POINTER(handle));
+			notfailed = g_slist_prepend (notfailed, (void *) args[i]);
 		} else {
-			failed = g_slist_prepend (failed, args[i]);
+			failed = g_slist_prepend (failed, (void *) args[i]);
 		}
 		i++;
 	}
@@ -216,7 +216,7 @@ main (int argc, char *argv[])
 					       GNOME_STOCK_BUTTON_CANCEL,
 					       NULL);
 		gnome_dialog_button_connect (GNOME_DIALOG (dialog),0,
-					    cancel_install, handles);
+					    G_CALLBACK(cancel_install), handles);
 		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
 		g_free (message);
 	}
